package com.jpsoft.picc.modules.base.service.impl;

import com.jpsoft.picc.modules.base.entity.JpCloudUser;
import com.jpsoft.picc.modules.base.service.JpCloudUserService;
import com.jpsoft.picc.modules.common.dto.MsgResult;
import com.jpsoft.picc.modules.common.utils.SignUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Component(value="jpCloudUserService")
public class JpCloudUserSeriveImpl implements JpCloudUserService {
    @Value("${jpcloud.findMemberByNamesUrl}")
    private String findMemberByNamesUrl;

    @Value("${jpcloud.findMemberUrl}")
    private String findMemberUrl;

    @Value("${jpcloud.platformCode}")
    private String platformCode;

    @Value("${jpcloud.platformKey}")
    private String platformKey;

    @Autowired
    private RestTemplate restTemplate;

    public <T> MsgResult<T> post(String url,HashMap<String,Object> searchParams){
        MsgResult<T> msgResult = null;

        try {
            ParameterizedTypeReference ref = new ParameterizedTypeReference<MsgResult<T>>() {
            };

            HttpHeaders headers = new HttpHeaders();
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            headers.setContentType(MediaType.parseMediaType("multipart/form-data;charset=UTF-8"));

            MultiValueMap<String, Object> paramMap = new LinkedMultiValueMap<>();

            for (String key : searchParams.keySet()) {
                paramMap.add(key, searchParams.get(key));
            }

            paramMap.add("sign", SignUtil.createSign(searchParams, platformKey));

            HttpEntity<MultiValueMap<String, Object>> formEntity = new HttpEntity<>(paramMap, headers);

            ResponseEntity<MsgResult<T>> responseEntity =
                    restTemplate.exchange(url, HttpMethod.POST, formEntity, ref);

            if (responseEntity.getStatusCode() == HttpStatus.OK) {
                msgResult = responseEntity.getBody();
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }

        return msgResult;
    }

    @Override
    public List<JpCloudUser> findUserByNames(List<String> nameList) {
        String names = nameList.stream().collect(Collectors.joining(","));

        HashMap<String,Object> searchParams = new HashMap<>();
        searchParams.put("names",names);
        searchParams.put("platCode",platformCode);

        MsgResult<List<JpCloudUser>> msgResult = post(findMemberByNamesUrl,searchParams);

        if(msgResult!=null && msgResult.isSuccess()) {
            return msgResult.getData();
        }

        return null;
    }

    @Override
    public List<JpCloudUser> findUser(String name, String realName, String telephone) {
        HashMap<String,Object> searchParams = new HashMap<>();
        searchParams.put("name",name);
        searchParams.put("realName",realName);
        searchParams.put("telephone",telephone);
        searchParams.put("platCode",platformCode);

        MsgResult<List<JpCloudUser>> msgResult = post(findMemberUrl,searchParams);

        if(msgResult!=null && msgResult.isSuccess()) {
            return msgResult.getData();
        }

        return null;
    }
}
